#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2020-08-15 10:29:56
# @Author  : Chenghao Mou (mouchenghao@gmail.com)


def interpolate_tuple(startcolor, goalcolor, steps):
    """
    Take two RGB color sets and mix them over a specified number of steps.  Return the list
    """
    # white

    R = startcolor[0]
    G = startcolor[1]
    B = startcolor[2]

    targetR = goalcolor[0]
    targetG = goalcolor[1]
    targetB = goalcolor[2]

    DiffR = targetR - R
    DiffG = targetG - G
    DiffB = targetB - B

    buffer = []

    for i in range(steps + 1):
        iR = int(R + (DiffR * i / steps))
        iG = int(G + (DiffG * i / steps))
        iB = int(B + (DiffB * i / steps))

        color = f"{iR},{iG},{iB}"
        buffer.append(color)

    return buffer


def interpolate(startcolor, goalcolor, steps):
    """
    wrapper for interpolate_tuple that accepts colors as html ("#CCCCC" and such)
    """
    start_tuple = list(map(int, startcolor.split(",")))
    goal_tuple = list(map(int, goalcolor.split(",")))

    return interpolate_tuple(start_tuple, goal_tuple, steps)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2020-03-15 15:09:54
# @Author  : Chenghao Mou (mouchenghao@gmail.com)


import argparse

from loguru import logger

from touchbar_lyric.netease_music import main

if __name__ == "__main__":  # pragma: no cover
    parser = argparse.ArgumentParser("TouchBar Lyric Script for BTT")
    parser.add_argument(
        "--app", default="Spotify", choices=["Spotify", "Music"], help="Music application. Spotify has a better support",
    )

    parser.add_argument("--rainbow", default=False, action="store_true", help="Rainbow backgroud colors")
    parser.add_argument("--minimal", default=False, action="store_true", help="Deprecated")
    parser.add_argument(
        "--traditional", default=False, action="store_true", help="Use traditional Chinese",
    )
    parser.add_argument("--verbose", action="store_true", help="Turn on debug mode", default=False)

    parser.add_argument("--bg", default="51,204,153", type=str, help="Background color in RGB")
    parser.add_argument("--fs", default=12, type=int, help="Font size")
    parser.add_argument("--fc", default="255,255,255", type=str, help="Font color in RGB")

    args = parser.parse_args()
    if args.verbose:
        logger.enable("touchbar_lyric")
        logger.enable("__main__")
    else:
        logger.disable("touchbar_lyric")
        logger.disable("__main__")

    logger.debug(args)

    main(
        app=args.app,
        minimal=args.minimal,
        rainbow=args.rainbow,
        background_color=args.bg,
        font_size=args.fs,
        font_color=args.fc,
        traditional=args.traditional,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2020-08-14 19:51:58
# @Author  : Chenghao Mou (mouchenghao@gmail.com)

import datetime
from collections import defaultdict

import requests
from cachier import cachier
from loguru import logger

from touchbar_lyric import Song


@cachier(stale_after=datetime.timedelta(days=7),)
def get_lyric(title: str, artists: str) -> str:
    """Retrieve lyric from gecimi.com based on title and artists.

    Parameters
    ----------
    title : str
        Name of the song
    artists : str
        Names of the artists

    Returns
    -------
    str
        Most poplular choice for the given title and artists

    """

    candidates = defaultdict(list)
    for artist in artists.split(","):
        try:
            r = requests.get(f"http://gecimi.com/api/lyric/{title}/{artist}".replace(" ", "%20")).json()
            code, result = r["code"], r["result"]
            # print(r)
        except Exception as e:
            logger.debug(e)
            code = 404
            result = []

        if code == 0:
            for datum in result:
                try:
                    candidates[(datum["song"], datum["artist_id"])].append(requests.get(datum["lrc"]).text)
                except Exception as e:
                    logger.debug(e)
                    continue
        else:
            logger.debug(code)

    if candidates:
        majority = sorted(candidates.keys(), key=lambda x: len(candidates[x]), reverse=True)[0]
        return candidates[majority][0]

    return None


def current(title: str, artists: str, timestamp: int, traditional: bool = False) -> str:
    lyric = get_lyric(title, artists)

    if lyric is None:
        return None

    return Song(title=title, artists=artists, lyric=lyric, traditional=traditional).current(timestamp=timestamp)


if __name__ == "__main__":
    from touchbar_lyric import get_info

    title, artists, timestamp, status, _ = get_info("Spotify")
    print(title, artists)
    print(current(title, artists, timestamp, False))
